# include<system.h>

extern void irq0();
extern void irq1();
extern void irq2();
extern void irq3();
extern void irq4();
extern void irq5();
extern void irq6();
extern void irq7();
extern void irq8();
extern void irq9();
extern void irq10();
extern void irq11();
extern void irq12();
extern void irq13();
extern void irq14();
extern void irq15();

/* This array is actually an array of function pointers. We use 
* this to handle custom IRQ handlers for a given IRQ */

void *irq_routines[16] =
{
    0,0,0,0,0,0,0,0,
    0,0,0,0,0,0,0,0
};

/* This installs a custom IRQ handler for the given IRQ */
void irq_install_handler(int irq, void(*handler)(struct regs *r))
{
    irq_routines[irq] = handler;
}

void irq_uninstall_handler(int irq)
{
    irq_routines[irq] = 0;
}
    
/* This is a function to remap IRQ0 to 15 to be remapped to
* IDT entries 32 to 47 */
void irq_remap(void)
{
    outportb(0x20, 0x11);
    outportb(0xA0, 0x11);
    outportb(0x21, 0x20);
    outportb(0xA1, 0x28);
    outportb(0x21, 0x04);
    outportb(0xA1, 0x02);
    outportb(0x21, 0x01);
    outportb(0xA1, 0x01);
    outportb(0x21, 0x0);
    outportb(0xA1, 0x0);
}

void irq_install()
{
    irq_remap();
    idt_set_gate(32,(unsigned)irq0,0x08,0x8E);
    idt_set_gate(33,(unsigned)irq1,0x08,0x8E);
    idt_set_gate(34,(unsigned)irq2,0x08,0x8E);
    idt_set_gate(35,(unsigned)irq3,0x08,0x8E);
    idt_set_gate(36,(unsigned)irq4,0x08,0x8E);
    idt_set_gate(37,(unsigned)irq5,0x08,0x8E);
    idt_set_gate(38,(unsigned)irq6,0x08,0x8E);
    idt_set_gate(39,(unsigned)irq7,0x08,0x8E);
    
    idt_set_gate(40,(unsigned)irq8,0x08,0x8E);
    idt_set_gate(41,(unsigned)irq9,0x08,0x8E);
    idt_set_gate(42,(unsigned)irq10,0x08,0x8E);
    idt_set_gate(43,(unsigned)irq11,0x08,0x8E);
    idt_set_gate(44,(unsigned)irq12,0x08,0x8E);
    idt_set_gate(45,(unsigned)irq13,0x08,0x8E);
    idt_set_gate(46,(unsigned)irq14,0x08,0x8E);
    idt_set_gate(47,(unsigned)irq15,0x08,0x8E);
    
    
}

/* The value of the argument struct regs *r is taken from the stack, and the appropriate
* contents of the registers eax, si..etc are initialized with
* the values that have been pushed onto the stack before calling the 
* interrupt handler function hence r->int_no will have the value 
* of the interrupt number pushed by push xx in the irqxx: part in start.asm
*/

void irq_handler(struct regs *r)        
{
    /* This is a blank function pointer */
    void (*handler)(struct regs *r);

    /* Find our if we have a custom handler to run for this
    * IRQ, and then finally, run it */
    handler = irq_routines[r->int_no - 32];
    if(handler)
    {
        handler(r);
    }
    
    /* If the IDT entry that was invoked was greater than 40 
    * meaning IRQ8 to 15 , then we need to send an EOI to 
    * the slave controller */
    if(r->int_no >= 40)
    {
        outportb(0xA0, 0x20);
    }
    
    /* In either case, we need to send an EOI to the master
    * interrupt controller too */
    outportb(0x20, 0x20);


}
