# include<system.h>

unsigned short *textmemptr;
int attrib = 0x0F;
int csr_x  = 0, csr_y = 0;

void scroll(void)
{
    unsigned blank, temp;
    blank = 0x20 | (attrib << 8);
    
    if(csr_y >= 25)
    {
        temp = csr_y - 25 + 1;
        memcpy(textmemptr, textmemptr+temp*80, (25-temp)*80*2);
        memsetw(textmemptr + (25-temp)*80,blank,80);
        csr_y = 25 - 1;
    }
}

void move_csr(void)
{
    unsigned temp;
    temp = csr_y * 80 + csr_x;
    outportb(0x3D4,14);
    outportb(0x3D5, temp >> 8);
    outportb(0x3D4, 15);
    outportb(0x3D5, temp);
}

void cls()
{
    unsigned blank;
    int i;
    blank = 0x20 | (attrib << 8);
    for(i=0;i<25;i++)
        memsetw(textmemptr+i*80, blank, 80);
    
    csr_x = 0;
    csr_y = 0;
    move_csr();
}

void putch(unsigned char c)
{
    unsigned short *where;
    unsigned att = attrib << 8;
    /* Handles a backspace by moving the cursor back one space */
    if(c == 0x08)
    {
        if(csr_x != 0) csr_x --;
    }
    /* Handles a tab by incrementing the cursor's x, but only
    *  to a point that will make it divisible by 8 */
    else if(c == 0x09)
    {
        csr_x = (csr_x + 8) & ~(8 - 1);
    }
    /* Handles a 'Carriage Return', which simply brings the
    *  cursor back to the margin */
    else if(c == '\r')
    {
        csr_x = 0;
    }
    /* We handle our newlines the way DOS and the BIOS do: we
    *  treat it as if a 'CR' was also there, so we bring the
    *  cursor to the margin and we increment the 'y' value */
    else if(c == '\n')
    {
        csr_x = 0;
        csr_y++;
    }
    /* Any character greater than and including a space, is a
    *  printable character. The equation for finding the index
    *  in a linear chunk of memory can be represented by:
    *  Index = [(y * width) + x] */
    else if(c >= ' ')
    {
        where = textmemptr + (csr_y * 80 + csr_x);
        *where = c|att;
        csr_x ++;
    }
    /*if cursor has reached the edge of the screen's width*/
    if (csr_x >= 80)
    {
        csr_x = 0;
        csr_y ++;
    }
    
    scroll();
    move_csr();
}

void puts(unsigned char* text)
{
    int i;
    for(i=0;i<strlen(text);i++)
    {
        putch(text[i]);
    }
}


void putNumber(int str)
{
    int number;
    int power;
    power = 10;
    number = 0;
    while(str/power != 0)
    	power = power * 10;
    power = power / 10;
    while(str != 0)
    {
    	number = str / power;
	    putch(number + '0');
    	str = str-(number*power);
    	power = power / 10;
    }
}

void puthex(u32int n)
{
    u32int tmp;

    puts("0x");

    char noZeroes = 1;

    int i;
    for (i = 28; i > 0; i -= 4)
    {
        tmp = (n >> i) & 0xF;
        if (tmp == 0 && noZeroes != 0)
        {
            continue;
        }
    
        if (tmp >= 0xA)
        {
            noZeroes = 0;
            putch(tmp-0xA+'a' );
        }
        else
        {
            noZeroes = 0;
            putch( tmp+'0' );
        }
    }
  
    tmp = n & 0xF;
    if (tmp >= 0xA)
    {
        putch(tmp-0xA+'a');
    }
    else
    {
        putch(tmp+'0');
    }

}

void settextcolor(unsigned char forecolor, unsigned char backcolor)
{
    attrib = (backcolor << 4) | (forecolor & 0x0F);
}

void init_video(void)
{
    textmemptr = (unsigned short*)0xB8000;
    cls();
}
